@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 2014-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,17 +13,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2020
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2020
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2015
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2020
-# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 
 import re
 import sys
@@ -32,35 +21,37 @@ from os.path import dirname, join
 from time import time
 
 from flask import request, render_template, redirect, make_response
+from urllib.parse import quote, unquote
 
 from rucio.api import authentication as auth, identity
 from rucio.api.account import account_exists, get_account_info, list_account_attributes
 from rucio.common.config import config_get, config_get_bool
+from rucio.common.extra import import_extras
 from rucio.core import identity as identity_core, vo as vo_core
 from rucio.db.sqla.constants import AccountType, IdentityType
 
-try:
-    from urllib import unquote, quote
-except ImportError:
-    from urllib.parse import unquote, quote
-
-
 if sys.version_info > (3, 0):
     long = int
     unicode = str
 
-escapefunc = None
-try:
+if sys.version_info > (3, 2):
     import html
-    escapefunc = html.escape
-except ImportError:
-    import cgi
-    escapefunc = cgi.escape
 
-try:
-    from onelogin.saml2.auth import OneLogin_Saml2_Auth
+    def html_escape(s, quote=True):
+        return html.escape(s, quote)
+else:
+    import cgi
+
+    def html_escape(s, quote=True):
+        return cgi.escape(s, quote)  # pylint: disable-msg=E1101
+
+
+EXTRA_MODULES = import_extras(['onelogin'])
+
+if EXTRA_MODULES['onelogin']:
+    from onelogin.saml2.auth import OneLogin_Saml2_Auth  # pylint: disable=import-error
     SAML_SUPPORT = True
-except:
+else:
     SAML_SUPPORT = False
 
 # check if there is preferred server side config for webui authentication
@@ -86,6 +77,9 @@ if not AUTH_TYPE:
 
 MULTI_VO = config_get_bool('common', 'multi_vo', raise_exception=False, default=False)
 
+# Additional error message that can have VO specific information for the user, e.g., support mailing list.
+ADDITIONAL_ERROR_MSG = config_get("webui", "additional_error_msg", raise_exception=False, default="")
+
 # excluded characters for injected JavaScript variables
 VARIABLE_VALUE_REGEX = re.compile(r"^[\w\- /=,.+*#()\[\]]*$", re.UNICODE)
 
@@ -93,45 +87,39 @@ VARIABLE_VALUE_REGEX = re.compile(r"^[\w\- /=,.+*#()\[\]]*$", re.UNICODE)
 # catch these from the webpy input() storage object
 # will allow to remove also lines around each use of select_account_name
 
-COOKIES = []
 
-
-def html_escape(s, quote=True):
-    return escapefunc(s, quote)
-
-
-def prepare_saml_request():
+def prepare_saml_request(environ, data):
     """
     TODO: Validate for Flask
     Prepare a webpy request for SAML
-    :param request: webpy request object
+    :param environ: Flask request.environ object
     :param data: GET or POST data
     """
-    if request.environ('mod_wsgi.url_scheme') == 'https':
-        return {
-            'https': 'on' if request.environ.get('modwsgi.url_scheme') == 'https' else 'off',
-            'http_host': request.environ.get('HTTP_HOST'),
-            'server_port': request.environ.get('SERVER_PORT'),
-            'script_name': request.environ.get('SCRIPT_NAME'),
-            'get_data': request.data,
+    if environ.get('mod_wsgi.url_scheme') == 'https':
+        ret = {
+            'https': 'on' if environ.get('modwsgi.url_scheme') == 'https' else 'off',
+            'http_host': environ.get('HTTP_HOST'),
+            'server_port': environ.get('SERVER_PORT'),
+            'script_name': environ.get('SCRIPT_NAME'),
             # Uncomment if using ADFS as IdP
             # 'lowercase_urlencoding': True,
-            'post_data': request.data
         }
+        if data:
+            ret['get_data'] = data
+            ret['post_data'] = data
+        return ret
 
     return None
 
 
-def add_cookies(request):
-    global COOKIES
+def add_cookies(response, cookie={}):
+    for int_cookie in cookie:
+        response.set_cookie(**int_cookie)
 
-    for cookie in COOKIES:
-        request.set_cookie(**cookie)
-
-    return(request)
+    return(response)
 
 
-def redirect_to_last_known_url():
+def redirect_to_last_known_url(cookie):
     """
     Checks if there is preferred path in cookie and redirects to it.
     :returns: redirect to last known path
@@ -139,7 +127,7 @@ def redirect_to_last_known_url():
     requested_path = request.cookies.get('rucio-requested-path')
     if not requested_path:
         requested_path = request.environ.get('REQUEST_URI')
-    resp = add_cookies(make_response(redirect(requested_path, code=303)))
+    resp = add_cookies(make_response(redirect(requested_path, code=303)), cookie)
 
     return resp
 
@@ -216,9 +204,9 @@ def get_token(token_method, acc=None, vo=None, idt=None, pwd=None):
         return None
     try:
         if pwd:
-            token = token_method(acc, idt, pwd, 'webui', request.environ.get('REMOTE_ADDR'), vo=vo).token
+            token = token_method(acc, idt, pwd, 'webui', request.environ.get('REMOTE_ADDR'), vo=vo).get('token')
         else:
-            token = token_method(acc, idt, 'webui', request.environ.get('REMOTE_ADDR'), vo=vo).token
+            token = token_method(acc, idt, 'webui', request.environ.get('REMOTE_ADDR'), vo=vo).get('token')
         return token
     except:
         return None
@@ -239,7 +227,7 @@ def validate_webui_token(from_cookie=True, session_token=None):
     if not valid_token_dict or not session_token:
         return None
     else:
-        valid_token_dict['token'] = session_token
+        valid_token_dict['token'] = session_token  # pylint: disable=E1137
         return valid_token_dict
 
 
@@ -263,10 +251,10 @@ def finalize_auth(token, identity_type, cookie_dict_extra=None):
     :param cookie_dict_extra: extra cookies to set, dictionary expected
     :returns: redirects to the final page or renders a page with an error message.
     """
-    global COOKIES
+    cookie = []
     valid_token_dict = validate_webui_token(from_cookie=False, session_token=token)
     if not valid_token_dict:
-        return render_template("problem.html", msg="It was not possible to validate and finalize your login with the provided token.")
+        return render_template("problem.html", msg="It was not possible to validate and finalize your login with the provided token: " + token)
     try:
         attribs = list_account_attributes(valid_token_dict['account'], valid_token_dict['vo'])
         accounts = identity.list_accounts_for_identity(valid_token_dict['identity'], identity_type)
@@ -275,18 +263,18 @@ def finalize_auth(token, identity_type, cookie_dict_extra=None):
             accvalues += acc + " "
         accounts = accvalues[:-1]
 
-        COOKIES.extend([{'key': 'x-rucio-auth-token', 'value': quote(token)},
-                        {'key': 'x-rucio-auth-type', 'value': quote(identity_type)},
-                        {'key': 'rucio-auth-token-created-at', 'value': str(long(time()))},
-                        {'key': 'rucio-available-accounts', 'value': quote(accounts)},
-                        {'key': 'rucio-account-attr', 'value': quote(dumps(attribs))},
-                        {'key': 'rucio-selected-account', 'value': quote(valid_token_dict['account'])},
-                        {'key': 'rucio-selected-vo', 'value': quote(valid_token_dict['vo'])}])
+        cookie.extend([{'key': 'x-rucio-auth-token', 'value': quote(token)},
+                       {'key': 'x-rucio-auth-type', 'value': quote(identity_type)},
+                       {'key': 'rucio-auth-token-created-at', 'value': str(long(time()))},
+                       {'key': 'rucio-available-accounts', 'value': quote(accounts)},
+                       {'key': 'rucio-account-attr', 'value': quote(dumps(attribs))},
+                       {'key': 'rucio-selected-account', 'value': quote(valid_token_dict['account'])},
+                       {'key': 'rucio-selected-vo', 'value': quote(valid_token_dict['vo'])}])
 
         if cookie_dict_extra:
             for key, value in cookie_dict_extra.items():
-                COOKIES.append({'key': key, 'value': value})
-        return redirect_to_last_known_url()
+                cookie.append({'key': key, 'value': value})
+        return redirect_to_last_known_url(cookie)
     except Exception:
         return render_template("problem.html", msg="It was not possible to validate and finalize your login with the provided token.")
 
@@ -356,12 +344,19 @@ def x509token_auth(data=None):
             vos_with_desc = get_vo_descriptions(ui_vo)
             return add_cookies(make_response(render_template("select_login_method.html", oidc_issuers=AUTH_ISSUERS, saml_support=SAML_SUPPORT, possible_vos=vos_with_desc)))
 
-    if MULTI_VO:
-        msg = "<br><br>Your certificate (%s) is not mapped to (possibly any) rucio account: %s at VO: %s." % (html_escape(dn), html_escape(ui_account), html_escape(ui_vo))
+    if not ui_account:
+        if MULTI_VO:
+            msg = "<br><br>Your certificate (%s) is not mapped to (possibly any) rucio account at VO: %s." % (html_escape(dn), html_escape(ui_vo))
+        else:
+            msg = "<br><br>Your certificate (%s) is not mapped to (possibly any) rucio account." % (html_escape(dn))
     else:
-        msg = "<br><br>Your certificate (%s) is not mapped to (possibly any) rucio account: %s." % (html_escape(dn), html_escape(ui_account))
-    msg += "<br><br><font color=\"red\">First, please make sure it is correctly registered in <a href=\"https://voms2.cern.ch:8443/voms/atlas\">VOMS</a> and be patient until it has been fully propagated through the system.</font>"
-    msg += "<br><br>Then, if it is still not working please contact <a href=\"mailto:atlas-adc-ddm-support@cern.ch\">DDM Support</a>."
+        if MULTI_VO:
+            msg = "<br><br>Your certificate (%s) is not mapped to (possibly any) rucio account: %s at VO: %s." % (html_escape(dn), html_escape(ui_account), html_escape(ui_vo))
+        else:
+            msg = "<br><br>Your certificate (%s) is not mapped to (possibly any) rucio account: %s." % (html_escape(dn), html_escape(ui_account))
+
+    if ADDITIONAL_ERROR_MSG:
+        msg += ADDITIONAL_ERROR_MSG
     if not ui_account:
         return render_template("problem.html", msg=msg)
     token = get_token(auth.get_auth_token_x509, acc=ui_account, vo=ui_vo, idt=dn)
@@ -436,7 +431,7 @@ def saml_auth(method, data=None):
     :returns: rendered final page or a page with error message
     """
     SAML_PATH = join(dirname(__file__), 'saml/')
-    req = prepare_saml_request()
+    req = prepare_saml_request(request.environ, data)
     samlauth = OneLogin_Saml2_Auth(req, custom_base_path=SAML_PATH)
     saml_user_data = request.cookies.get('saml-user-data')
     if not MULTI_VO:
@@ -586,8 +581,10 @@ def oidc_auth(account, issuer, ui_vo=None):
               'webhome': realhome + '/oidc_final'}
     auth_url = auth.get_auth_oidc(account, vo=ui_vo, **kwargs)
     if not auth_url:
-        return render_template("problem.html", msg="It was not possible to get the OIDC authentication url from the Rucio auth server. "
-                                + "In case you provided your account name, make sure it is known to Rucio.")   # NOQA: W503
+        return render_template("problem.html", msg=("It was not possible to get the OIDC authentication url "
+                                                    "from the Rucio auth server. "
+                                                    "In case you provided your account name, make sure it is "
+                                                    "known to Rucio."))
     return redirect(auth_url, code=303)
 
 
@@ -597,16 +594,17 @@ def authenticate(template, title):
     :param template: the template name that should be rendered
     :returns: rendered final page or a page with error message
     """
-    global AUTH_ISSUERS, SAML_SUPPORT, AUTH_TYPE, COOKIES
+    global AUTH_ISSUERS, SAML_SUPPORT, AUTH_TYPE
+    cookie = []
     valid_token_dict = validate_webui_token()
     if not valid_token_dict:
-        COOKIES.append({'key': 'rucio-requested-path', 'value': request.environ.get('REQUEST_URI')})
+        cookie.append({'key': 'rucio-requested-path', 'value': request.environ.get('REQUEST_URI')})
     else:
         return access_granted(valid_token_dict, template, title)
 
     # login without any known server config
     if not AUTH_TYPE:
-        return add_cookies(make_response(render_template("select_login_method.html", oidc_issuers=AUTH_ISSUERS, saml_support=SAML_SUPPORT)))
+        return add_cookies(make_response(render_template("select_login_method.html", oidc_issuers=AUTH_ISSUERS, saml_support=SAML_SUPPORT)), cookie)
     # for AUTH_TYPE predefined by the server continue
     else:
         if AUTH_TYPE == 'userpass':
