@@ -8,6 +8,7 @@
  * Authors:
  * - Thomas Beermann, <thomas.beermann@cern.ch>, 2015-2018
  * - Eric Vaandering, <ewv@fnal.gov>, 2020
+ * - Martin Barisits, <martin.barisits@cern.ch>, 2021
  */
 
 var selected_dids = [];
@@ -786,6 +787,99 @@ create_request_container = function(dids) {
     return [{'scope': scope, 'name': name}]
 }
 
+create_request_dataset = function(dids) {
+    scope = "";
+    r.get_account_info({
+        account: account,
+        async: false,
+        success: function(data) {
+            if (data['account_type'] == 'USER') {
+                scope = 'user.' + account;
+            } else if (data['account_type'] == 'GROUP') {
+                scope = 'group.' + account;
+            }
+        },
+        error: function(jqXHR, textStatus, errorThrown) {
+            console.log(jqXHR);
+        }
+    });
+    var now = new Date();
+    var date = now.getFullYear() + '-';
+    if ((now.getMonth() + 1) < 10) {
+        date += '0' + (now.getMonth() + 1);
+    } else {
+        date += (now.getMonth() + 1);
+    }
+    date += '-';
+    if (now.getDate() < 10) {
+        date += '0' + now.getDate();
+    } else {
+        date += now.getDate();
+    }
+    var hour = now.getHours();
+    if (hour < 10) {
+        hour = '0' + hour;
+    }
+    date += '_' + hour;
+    var minutes = now.getMinutes();
+    if (minutes < 10) {
+        minutes = '0' + minutes;
+    }
+    date += '-' + minutes;
+    var seconds = now.getSeconds();
+    if (seconds < 10) {
+        seconds = '0' + seconds;
+    }
+    date += '-' + seconds;
+
+    name = scope + ".r2d2_request." + date;
+
+    r.add_did({
+        'scope': scope,
+        'name': name,
+        'type': 'DATASET',
+        'async': false,
+        success: function(data) {
+        },
+        error: function(jqXHR, textStatus, errorThrown) {
+            console.log(jqXHR);
+            html_check = '<font color="red">There was an error creating the request dataset.</font>';
+            $('#submit_progress').html('');
+            $("#error_msg").html(html_check);
+        }
+    });
+    r.attach_dids({
+        'scope': scope,
+        'name': name,
+        'dids': dids,
+        'async': false,
+        success: function(data) {
+        },
+        error: function(jqXHR, textStatus, errorThrown) {
+            console.log(jqXHR);
+            html_check = '<font color="red">There was an error adding the DIDs to the request dataset.</font>';
+            $('#submit_progress').html('');
+            $("#error_msg").html(html_check);
+        }
+    });
+    r.set_status({
+        'scope': scope,
+        'name': name,
+        'open': false,
+        'async': false,
+        success: function(data) {
+        },
+        error: function(jqXHR, textStatus, errorThrown) {
+            console.log(jqXHR);
+            html_check = '<font color="red">There was an error closing the request dataset.</font>';
+            $('#submit_progress').html('');
+            $("#error_msg").html(html_check);
+        }
+    });
+
+    return [{'scope': scope, 'name': name}]
+}
+
 
 create_rules = function(approval) {
     var html_ok = '<div class="row">';
@@ -825,16 +919,44 @@ create_rules = function(approval) {
     if (selected_options['sample'] != null) {
         create_sample();
     }
+
     selected_dids = storage.get('selected_dids');
 
     var request_container = null;
+    did_datasets = [];
+    did_files = [];
     if ((selected_dids.length > 1) && (approval)) {
-        request_container = create_request_container(selected_dids);
+        $.each(selected_dids, function(index, did) {
+            r.get_did({
+                'scope': did['scope'],
+                'name': did['name'],
+                'dynamic': true,
+                success: function(data){
+                    if (data['did_type'] == 'FILE'){
+                        did_files.push({'scope': did['scope'], 'name': did['name']});
+                    } else if (data['did_type'] == 'DATASET'){
+                        did_datasets.push({'scope': did['scope'], 'name': did['name']});
+                    }
+                }
+            });
+        });
+    }
+
+    if (did_datasets.length != 0 && did_files.length != 0){
+        var request_dataset = create_request_dataset(did_files);
+        did_datasets.push({'scope': request_dataset[0]['scope'], 'name': request_dataset[0]['name']});
+        request_container = create_request_container(did_datasets);
+    } else if (did_datasets.length != 0 && did_files.length == 0) {
+        request_container = create_request_container(did_datasets);
+    } else if (did_datasets.length == 0 && did_files.length != 0){
+        var request_dataset = create_request_dataset(did_files);
     }
 
     rse_expr = storage.get('rse_expr');
     if (request_container) {
         options['dids'] = request_container;
+    } else if (request_dataset) {
+        options['dids'] = request_dataset;
     } else {
         options['dids'] = selected_dids;
     }
@@ -886,7 +1008,7 @@ create_rules = function(approval) {
         error_details = JSON.parse(jqXHR.responseText);
         if (error_details['ExceptionClass'] == 'DuplicateRule') {
             html_check = '<font color="red">The rule that you are trying to create already exists, please go back and choose another DID or RSE.</font>';
-        } else if (error_details['ExceptionClass'] == 'RSEBlacklisted') {
+        } else if (error_details['ExceptionClass'] == 'RSEWriteBlocked') {
             html_check = '<font color="red">The RSE that you chose is currently not available for writing. Please check <a href="http://atlas-agis.cern.ch/agis/ddmblacklisting/list/" target="_blank">here</a> for more information.</font>';
         } else {
             html_check = '<font color="red">' + error_details['ExceptionClass'] + ': ' + error_details['ExceptionMessage'] + '</font>';
@@ -1202,10 +1324,10 @@ create_summary = function(approval) {
 
 clear_form = function() {
     tour_alert = storage.get('tour_alert');
-    jira_alert = storage.get('jira_alert');
+    gh_issue_alert = storage.get('gh_issue_alert');
     storage.removeAll();
     storage.set('tour_alert', tour_alert);
-    storage.set('jira_alert', jira_alert)
+    storage.set('gh_issue_alert', gh_issue_alert)
     $("#pattern_input").val("");
     $("#rse_input").val("");
     $("#grouping_dataset".toLowerCase()).prop( "checked", true );
@@ -1285,15 +1407,15 @@ $(document).ready(function() {
         if (event.target.id == 'tour_alert') {
             storage.set('tour_alert', false);
         }
-        if (event.target.id == 'jira_alert') {
-            storage.set('jira_alert', false);
+        if (event.target.id == 'gh_issue_alert') {
+            storage.set('gh_issue_alert', false);
         }
     });
     if (storage.get('tour_alert') == false) {
         $("#tour_alert").hide();
     }
-    if (storage.get('jira_alert') == false) {
-        $("#jira_alert").hide();
+    if (storage.get('gh_issue_alert') == false) {
+        $("#gh_issue_alert").hide();
     }
 
     $('#check_sample').click(function() {
