@@ -25,12 +25,12 @@ function RucioClient(token, account, vo) {
                     'X-Rucio-VO': this.vo,
                     'X-Rucio-Script': this.script}
     host = window.location.host;
-    this.url = 'https://' + host + ':443/proxy';
-    this.authurl = 'https://' + host + ':443/auth';
+    this.url = 'https://' + host + '/proxy';
+    this.authurl = 'https://' + host + '/auth';
     if (window.location.href.includes('/ui/')){
-        this.authurl = 'https://' + host + ':443/ui/auth';
+        this.authurl = 'https://' + host + '/ui/auth';
     }
-    this.dumps = 'https://' + host + ':443/dumpsproxy';
+    this.dumps = 'https://' + host + '/dumpsproxy';
 };
 
 /* --- utility function --- */
@@ -59,9 +59,9 @@ check_token = function() {
             $.cookie('x-rucio-auth-token', "", { expires: -1 });
             $.cookie('rucio-requested-path', window.location.href, { expires: 120 , path: '/'});
             if (window.location.href.includes('/ui/')){
-                window.location.href = 'https://' + window.location.host + ':443/ui/auth';
+                window.location.href = 'https://' + window.location.host + '/ui/auth';
             } else {
-                window.location.href = 'https://' + window.location.host + ':443/auth';
+                window.location.href = 'https://' + window.location.host + '/auth';
             }
         } else {
             r.get_auth_token_x509({
@@ -865,7 +865,7 @@ RucioClient.prototype.list_replicas = function(options) {
     check_token();
     var url = this.url + '/replicas/list/';
     if (options.async == null) { options.async = true; }
-    data = {'dids': [{'scope': options.scope, 'name': options.name}], 'unavailable': true, 'all_states': true }
+    data = {'dids': [{'scope': options.scope, 'name': options.name}], 'unavailable': true, 'all_states': true, 'ignore_availability': true}
     if (options.schemes != null) {
         data['schemes'] = options.schemes;
     }
@@ -1684,16 +1684,18 @@ RucioClient.prototype.send_trace = function(options) {
 /* --- rucio dumps methods --- */
 
 /* list all subscriptions from dumps */
-RucioClient.prototype.list_subscription_rules_state_from_dumps = function(options) {
+RucioClient.prototype.list_subscription_rules_state_real_time = function(options) {
     check_token();
-    var url = this.dumps + '/subscription_states/' + options.date + '/' + options.hour + options.minutes + '.lst';
+    // replace root with actual account
+    var url = this.url + '/subscriptions/' + options.account + '/Rules/States';
     jQuery.ajax({url: url,
-                 headers: {'X-Rucio-Script': this.script},
+                 headers: {...this.headers },
                  async: false,
                  crossDomain: true,
+                 dataType: 'text',
                  success: function(data)
                  {
-                     options.success(data);
+                    options.success(parse_json_stream(data));
                  },
                  error: function(jqXHR, textStatus, errorThrown)
                  {
